/* 
  Kitchen Timer for ikkeiplexing shield by ikkei
  http://blog.goo.ne.jp/jh3kxm
  
  SW2: minute
  SW3: 10 second
  SW6: start/stop
  SW7: clear
  
  History:
        2010-12-16 - V1.0 Kitchen Timer by ikkei
        2013-10-26 - update class library
        
*/

#include <MsTimer2.h>
#include <ikkeiplexing.h>

ikkeiplexing iLedSign;

#define SW1  0x80
#define SW2  0x40
#define SW3  0x20
#define SW4  0x10
#define SW5  0x08
#define SW6  0x04
#define SW7  0x02
#define SW8  0x01

#define SW_MINUTE SW2
#define SW_TENSEC SW3
#define SW_START  SW7
//#define SW_START  SW1 // for Titchaino uni
#define SW_CLEAR  SW1

#define FRAME  0x01
#define TENMS  0x02
#define ALARM_PERIOD 10

const byte FIG[10][4] = {
{ 0x7C, 0x82, 0x82, 0x7C}, // "0"  
{ 0x00, 0x84, 0xFE, 0x80}, // "1"
{ 0xC4, 0xA2, 0x92, 0x8C}, // "2"
{ 0x44, 0x82, 0x92, 0x6C}, // "3"
{ 0x38, 0x24, 0xFE, 0x20}, // "4"
{ 0x5E, 0x92, 0x92, 0x62}, // "5"
{ 0x7C, 0x92, 0x92, 0x64}, // "6"
{ 0x06, 0xC2, 0x32, 0x0E}, // "7"
{ 0x6C, 0x92, 0x92, 0x6C}, // "8"
{ 0x4C, 0x92, 0x92, 0x7C}  // "9"
};

const byte ALED[2][16] = {
{ 0xFF, 0x81, 0xBD, 0xA5, 0xA5, 0xA5, 0xA5, 0xA5, 0xA5, 0xA5, 0xA5, 0xA5, 0xA5, 0xBD, 0x81, 0xFF},
{ 0x00, 0x7E, 0x42, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x42, 0x7E, 0x00}
};

typedef enum {
  SET_TIME, COUNT_DOWN, PAUSE, ALARM
} State;

State state;

//enum state{ set_time, count_down, pause, alarm };
//enum state  state;
uint8_t event;
byte set_minute = 0;
byte set_tensec = 0;
byte count_minute = 0;
byte count_tensec = 0;
byte count_second = 0;
byte ten_msec = 0;
byte clock = 0;
byte alarm_count = 0;
byte alarm_flash = 0;

void DisplayAlarm( byte data ){
  for (byte x=0; x<16; x++){
    for (byte y=0; y<8; y++){
      iLedSign.Set(x, y, bitRead( ALED[ data ][x],y));
    }
  }
}

void DisplayLED( byte data1, byte data2, byte data3 ){
  for (byte x=0; x<4; x++){
    for (byte y=0; y<8; y++){
      iLedSign.Set(x,    y, bitRead( FIG[ data1 ][x],y));
      iLedSign.Set(x+7,  y, bitRead( FIG[ data2 ][x],y));
      iLedSign.Set(x+12, y, bitRead( FIG[ data3 ][x],y));
    }
  }
  iLedSign.Set(5,1,1);
  iLedSign.Set(6,0,1);
}

	
void set_time_state(void){
  DisplayLED(set_minute, set_tensec, 0);
  if ((event & SW_MINUTE) != 0){
    set_minute++;
    if (set_minute > 9){
      set_minute = 0;
    }
  } else if ((event & SW_TENSEC) != 0){
    set_tensec++;
    if (set_tensec > 5){
      set_tensec =0;
    }
  } else if ((event & SW_START) != 0){
    if ((set_minute != 0)||(set_tensec != 0)){
      count_minute = set_minute;
      count_tensec = set_tensec;
      count_second = 0;
      clock = 0;
      state = COUNT_DOWN;
    }
  } else if ((event & SW_CLEAR) != 0){
    set_minute = 0;
    set_tensec = 0;
  }
}

void count_down_state(void){
    //DisplayLED(count_minute, count_tensec, count_second);
  clock++;
  if (clock >= 100){
    clock = 0;
    if (count_second > 0){
      count_second--;
    } else {
      count_second = 9;
      if (count_tensec > 0){
        count_tensec--;
      } else {
        count_tensec = 5;
        if (count_minute > 0){
          count_minute--;
        }
      }
    }
    DisplayLED(count_minute, count_tensec, count_second);
    if ((count_second == 0)&&(count_tensec == 0)&&(count_minute == 0)){
      alarm_count = ALARM_PERIOD;
      state = ALARM;
    }
  }
  if ((event & SW_START) != 0){
    state = PAUSE;
  }
}

void pause_state(void){
  if ((event & SW_START) != 0){
    state = COUNT_DOWN;
  } else if ((event & SW_CLEAR) != 0){
    set_minute = 0;
    set_tensec = 0;
    state = SET_TIME;
  }
}


void beep(){
  uint8_t s = 0;
  for (uint8_t i = 0; i<200; i++){
    s ^= 1;
    digitalWrite(13, s);
    delayMicroseconds(120);
  }
  delay(20);
}


void alarm_state(void){
  alarm_count--;
  if (alarm_count == 0){
    beep();
    alarm_count = ALARM_PERIOD;
    alarm_flash ^= 1;
    DisplayAlarm(alarm_flash);
  }
  if (((event & SW_MINUTE) != 0)||((event & SW_TENSEC) != 0)||((event & SW_START) != 0)||((event & SW_CLEAR) != 0)){
    iLedSign.Clear();
    state = SET_TIME;
  }
}

void setup() {
  iLedSign.Init(IKKEIPLEXING, SINGLE_BUFFER);
  set_minute = 0;
  set_tensec = 0;
  state = SET_TIME;
}

void loop(){
  if ((iLedSign.GetPeriod() & TENMS) != 0){
    event = iLedSign.GetSW(SW_EDGE);
    switch (state){
      case SET_TIME:    set_time_state();    break;
      case COUNT_DOWN:  count_down_state();  break;
      case PAUSE:       pause_state();       break;
      case ALARM:       alarm_state();       break;
    }
  }
}

